## R code corresponding to the analysis of Martnez et al.:
## "Mortality of NBA players: Risk factors and comparison with the general
##  US population"
## ============================================================================
load("MartinezEtAl.RData")

## Data frames
## -----------
#  mortality: Mortality data of the US population from 2000 to 2014 (Dim: 83 x 31)
#  nbadat: Data of all active and former NBA players (Dim: 3985 x 65)
#  nbadatBW: Subset of African-American and white NBA players (Dim: 3858 x 63)
#  nbaFormer: Data of all former NBA players (prior to 2014/15) (Dim: 3504 x 64)
#  nbaFormerBW: Subset of former african-american and white NBA players (Dim: 3425 x 63)


### ===========================================================================
### Table 1 ===================================================================
### ===========================================================================
library(compareGroups)
### Active vs Former
res1 <- compareGroups(state ~ pos + etni + place2 + lefthanded + ageleft +
                      ageend + cms2 + kilos + g, data = nbadat, type = 3)
restab1 <- createTable(res1, show.all = TRUE, show.p.overall = FALSE)
restab1

## Alive vs dead (among former NBA players)
res2 <- compareGroups(cens ~ pos + etni + place2 + lefthanded + ageleft +
                      ageend + cms2 + kilos + g, data = nbaFormer, type = 3)
restab2 <- createTable(res2, show.all = TRUE, show.p.overall = FALSE)
restab2

detach(package:compareGroups)

### ===========================================================================
### Figure 1 ==================================================================
### ===========================================================================
library(survival)
library(rms)

windows(width = 9, height = 6)
par(las = 1, font.lab = 2, font.axis = 2, font = 2, mar = c(5, 4, 2, 2))
survplot(npsurv(Surv(survtime, cens) ~ 1, nbaFormer), lwd = 3,
         xlab = "Years after NBA career", time.inc = 5, col.fill = grey(0.75),
         ylab = "Estimated survival probability")
segments(0, 0.5, 70, 0.5, lwd = 3)


### ===========================================================================
### Figure 2 ==================================================================
### ===========================================================================
windows(width = 9, height = 6)
par(las = 1, font.lab = 2, font.axis = 2, font = 2, mar = c(5, 4, 2, 2))
survplot(npsurv(Surv(ageleft, ageright, cens) ~ 1, nbadat), lwd = 3,
         xlab = "Age at death",
         time.inc = 5, col.fill = grey(0.75),
         ylab = "Estimated survival probability")
segments(0, 0.5, 100, 0.5, lwd = 3)


### ===========================================================================
### Table 2 (Cox models) ======================================================
### ===========================================================================
## Model 1
nbamod <- coxph(Surv(survtime, cens) ~ etni + cms2 + ageend + fins, nbaFormerBW)
summary(nbamod)

## Checking for proportional hazards assumption
cox.zph(nbamod)
windows(width = 12, height = 12)
par(mfrow = c(2, 2), font = 2, font.lab = 4, font.axis = 2, las = 1, pch = 16)
plot(cox.zph(nbamod), lwd = 3, col = 2)

## Model 2
nbamodLT <- coxph(Surv(ageleft, ageright, cens) ~ etni + cms2 + from, nbadatBW)
summary(nbamodLT)

## Checking for proportional hazards assumption
cox.zph(nbamodLT)
windows(width = 12, height = 12)
par(mfrow = c(2, 2), font = 2, font.lab = 4, font.axis = 2, las = 1, pch = 16)
plot(cox.zph(nbamodLT), lwd = 3, col = 2)


### ===========================================================================
### Table 3 (Estimated median ages) ===========================================
### ===========================================================================
newdat <- data.frame(etni = levels(nbadatBW$etni),
                     cms2 = 200, from = c(rep(seq(1950, 2010, 10), each = 2)))
survfit(nbamodLT, newdata = newdat)


### ===========================================================================
### Figure 3 (Standard Mortality Ratios) ======================================
### ===========================================================================
library(data.table)
library(epitools)

SMRs <- vector("list", 15)
names(SMRs) <- 2000:2014
for(i in seq_along(2000:2014)) {
  SMRs[[i]] <- vector("list", 2)
  names(SMRs[[i]]) <- c("AfricanAmerican", "White")
  SMRs[[i]][["AfricanAmerican"]] <- SMRs[[i]][["White"]] <- numeric(3)
}

years <- 2000:2014
for(y in seq_along(2000:2014)) {
  cat("\n")
  cat("Year", years[y], fill = TRUE)
  # Age distribution by January 1
  jan1 <- paste0(years[y], "-1-1")
  nbadatBW$age <- as.vector(trunc(difftime(jan1, nbadatBW$birthdate)/365.25))

  # Excluded those with later debut and death before. Age is computed, hence,
  #   for active players on January 1 and alive former players.
  nbadatBW$age <- with(nbadatBW, ifelse(debut > as.Date(jan1) |
                                        dateevent < as.Date(jan1), NA, age))
  # New variable: Death in current year
  nbadatBW$deathyr <- as.numeric(year(nbadatBW$dateevent) == years[y] &
                                 nbadatBW$cens == 1)

  # Separate tables for African-American and white players
  tabWDeaths <- with(subset(nbadatBW, etni == "White"), table(age, deathyr))
  tabBDeaths <- with(subset(nbadatBW, etni == "Black"), table(age, deathyr))

  # Auxiliary data frames
  auxW <- data.frame(age = as.numeric(rownames(tabWDeaths)),
                     nkW = rowSums(tabWDeaths), mkW = tabWDeaths[, 2])
  auxB <- data.frame(age = as.numeric(rownames(tabBDeaths)),
                     nkB = rowSums(tabBDeaths), mkB = tabBDeaths[, 2])

  # Mortality rates
  mrates <- merge(auxW, auxB, all = TRUE)

  # Merging with death rates of current year
  auxvs <- c("age", paste0("prob", years[y], "M", c("white", "black")))
  mrates <- merge(mrates, mortality[, auxvs], all = TRUE)
  names(mrates)[6:7] <- c("probWhite", "probBlack")

  # Computation of the SMRs
  smrW <- with(na.omit(mrates[, c(1:3, 6)]),
               round(ageadjust.indirect(mkW, nkW, stdcount = NULL, stdpop = NULL,
                                        stdrate = probWhite)$sir, 2))
  smrB <- with(na.omit(mrates[, c(1, 4, 5, 7)]),
               round(ageadjust.indirect(mkB, nkB, stdcount = NULL, stdpop = NULL,
                                        stdrate = probBlack)$sir, 2))

  # Results
  cat("White (ex-) players: SMR = ", smrW[3], "; 95%-CI: [", smrW[4], ", ",
      smrW[5], "].", sep = "", fill = TRUE)
  cat("African-American (ex-) players: SMR = ", smrB[3], "; 95%-CI: [",
      smrB[4], ", ", smrB[5], "].", sep = "", fill = TRUE)

  # Saving results
  SMRs[[y]][["White"]] <- smrW[3:5]
  SMRs[[y]][["AfricanAmerican"]] <- smrB[3:5]

  names(SMRs[[y]][["AfricanAmerican"]]) <- names(SMRs[[y]][["White"]]) <-
                                   c("SMR", "95%Low", "95%Upp")

  nbadatBW$age <- nbadatBW$deathyr <- NULL
  rm(jan1, tabWDeaths, tabBDeaths, auxW, auxB,auxvs)
}
rm(y)

## Graphical representation
## ------------------------
smrs <- data.frame(matrix(unlist(SMRs), nc = 6, byrow = TRUE))
rownames(smrs) <- years
names(smrs) <- paste0(rep(c("bl", "wh"), each = 3), c("smr", "low95", "upp95"))

windows(width = 9)
par(las = 1, font = 2, font.lab = 2, font.axis = 4, xlog = TRUE,
    mar = c(5, 4, 2, 2))
with(smrs, plot(years - 0.1, blsmr, pch = 16, xlab = "Year", ylab = "SMR",
                ylim = c(0.2, 1.4), log = "y", cex = 1.5, type = "b", lwd = 2))
points(years + 0.1, smrs$whsmr, pch = 15, cex = 1.5, col = 4, type = "b",
       lwd = 2)
legend("bottomleft", c("African-American", "White"), lwd = 3, pch = c(16, 15),
       col = c(1, 4), bty = "n", cex = 1.3)
axis(1, at = seq(2001, 2014, 2))
abline(h = 1, lty = 2, lwd = 2)
